
import subprocess
import json
import logging
from typing import Optional, List, Callable, Union
from enum import Enum, auto
from dataclasses import dataclass

from .flatpak import in_flatpak, prepend_flatpak


logger = logging.getLogger(__name__)


@dataclass
class Image:
    image: str
    digest: str


@dataclass
class State:
    booted: Image
    staged: Optional[Image]
    rollback_image: Optional[Image]
    is_rollback_queued: bool


def get_state() -> Optional[State]:
    logger.debug("Getting bootc state info")
    args = [
                "systemd-run", "--pipe",
                "bootc", "status", "--json"
    ]
    if in_flatpak():
        args = prepend_flatpak(args)
    try:
        proc_stdout = subprocess.run(
            args,
            stdout=subprocess.PIPE
        ).stdout.decode("utf-8")
        result = json.loads(proc_stdout)
    except:
        return None

    logger.debug(result)

    booted_data = result["status"]["booted"]
    booted_image = Image(
        image=booted_data["image"]["image"]["image"],
        digest=booted_data["image"]["imageDigest"]
    )

    staged_data = result["status"]["staged"]
    try:
        staged_image = Image(
            image=staged_data["image"]["image"]["image"],
            digest=staged_data["image"]["imageDigest"]
        )
    except (KeyError, TypeError):
        staged_image = None

    rollback_data = result["status"]["rollback"]
    try:
        rollback_image = Image(
            image=rollback_data["image"]["image"]["image"],
            digest=rollback_data["image"]["imageDigest"]
        )
    except (KeyError, TypeError):
        rollback_image = None
    is_rollback_queued = result["status"]["rollbackQueued"]

    state = State(
        booted=booted_image,
        staged=staged_image,
        rollback_image=rollback_image,
        is_rollback_queued=is_rollback_queued,
    )

    return state


def get_available_update() -> Union[Exception, Optional[Image]]:
    """
    Returns:
        Exception: Error has occurred
        None: No updates available
        Image: The update available
    """
    args = [
                "systemd-run", "--pipe",
                "bootc", "upgrade", "--check"
    ]
    if in_flatpak():
        args = prepend_flatpak(args)
    logger.debug("Getting available update info")
    try:
        proc_stdout = subprocess.run(
            args,
            stdout=subprocess.PIPE
        ).stdout.decode("utf-8")
    except Exception as e:
        return e
    result = proc_stdout.splitlines()

    if "No changes in:" in proc_stdout:
        return None

    try:
        image = result[0].split("docker://")[1]
    except (IndexError, TypeError) as e:
        return e

    try:
        digest = result[2].split("Digest: ")[1][0:14]
    except (IndexError, TypeError) as e:
        return e

    available_update = Image(
        image=image,
        digest=digest
    )
    return available_update


def start_upgrade(callback: Callable):
    logger.debug("Starting upgrade")

    args = [
            "systemd-run", "--pipe",
            "bootc", "upgrade"
    ]
    if in_flatpak():
        args = prepend_flatpak(args)

    proc = subprocess.run(
        args,
        capture_output=True
    )

    callback(proc)

